Contribuer au projet
===

Documentation haut-niveau, décrivant le projet du point de vue développeur de façon globale, sans rentrer dans le détail du code. 

Une documentation plus précise, générée par JSDoc à partir des commentaires dans le code-même, [est disponible dans ./doc](./doc/index.html). Pour la générer, exécuter `npm run doc`.

## Description et actualité des branches

#### Migrations Knex

Pour `./db/migrations`, il faut utiliser migrations_v3 (la dernière) (c'est logique). Ca s'appelle `migrations_v3` et pas `migrations` parce que je ne voulais pas supprimer les deux autres versions sans confirmer auprès des autres devs que c'était ok. -- kadabra

### _stable_

- déploiement de l'API pour les développeurs du front

### _master_

- une branche où on fait des petits commits pour merge dans stable

### redo-connectors

- branche sur laquelle akka vodol fait tout son travail : implémentation des resolvers pour le nouveau schéma propre (le schéma "_wish_list"). Comme ça, il peut faire d'éventuelles modifications du schéma sans faire chier les autres, et ensuite on valide ensemble pour merge.

### LDAP-refacto

- TODO (@hawkspar tu peux expliquer ici a quoi sert cette branche  ? et le cas échéant, la merge dans master ?)

### unit-tests

- TODO (@akka vodol tu peux expliquer ici a quoi sert cette branche ? et le cas échéant, la merge dans master ?)

## Structure du dossier

La structure générale du projet est détaillée ci-dessous ; pas d'inquiétude, la plupart des fichiers .js sont aussi extensivement documentés dans la doc générée par JSDoc (ou plus bas si vous lisez depuis JSDoc).

Divers fichiers de configuration à la racine du projet :
- `.eslintignore` : dit à eslint de ne pas linter ces fichiers.
- `.eslintrc.json` : config pour eslint. Le plus susceptible d'être modifié est le champ rules. env spécifie l'environnement, et donc quelle syntaxe est utilisée.
- `.gitignore` : dit à git de ne pas track ces fichiers.
- `package-lock.json` : spécifie à npm les dépendances du projet avec leur version précise. équivalent de pip freeze pour flask/django.
- `package.json` : spécifie à npm les dépendances du projet, mais aussi l'auteur, la license, le repo git... bref tout ce qui peut être intéressant pour [npm](https://www.npmjs.com/package/shitpost). Spécifie aussi les scripts à utiliser quand on utilise la commande `npm run (blablabla)`.
- `configfile_doc.json` : config pour JSDoc, l'outil de génération automatique de documentation
- autres fichiers relatifs à des modules particuliers ou à la gestion de paquets
  - tsconfig.json : pour typescript
  - webpack.config.js
  - .gitlab-ci.yml : pour le CI (continuous integration) de gitlab
  - .gitattributes : ??
  - .dockerignore : pour Docker
  - Dockerfile : pour déployer dans un conteneur Docker
- `config.json` : données "sensibles" spécifique au projet à priori non partageable (adresse du LDAP, mots de passe)

Les dossiers à la racine du projet :
- [`assets/`](./assets) : ressources. on y met les fichiers statiques, e.g. les images
  - images et divers (dont l'essentielle favicon)
- [`build/`](./build) : output de webpack
  - bundle.js est un monstrueux fichier généré par webpack, qui compile et optimise tout notre code de `src/`
- [`db/`](./db) : fichiers utilisés par Knex.js, le traducteur javascript/SQL
  - `knexfile.js` : fichier de config de la commande `knex <...>` *en CLI*
  - `knex_router.js` : exporte l'objet `knex` *utilisé dans le code javascript*. En fait, ne fait que charger la config déjà définie dans `knexfile.js`
  - [`migrations/`](./db/migrations) stocke l'ensemble des migrations Knex
  - [`seeds/`](./db/seeds) : stocke les seeds Knex (scripts de génération de contenu de la BDD)
- [`doc/`](./doc) : documentation bas-niveau, générée par JSDoc à partir des commentaires dans le code-même. Ensemble de pages html et leurs fichiers associés. *index.html* permet de naviguer sereinement sur toute la doc JSDoc
- [`node_modules/`](./node_modules) : les dépendances du projet, téléchargées automatiquement par npm grâce à leur liste dans `package.json`
- [`src/`](./src) : code source de l'application Node.js du backend
  - [`adminview/`](./src/adminview) : code source d'une "mini-application" gérant l'interface administrateur du backend, utilisée lorsque le backend reçoit des requêtes HTTP GET
    - `css/` : les stylesheets CSS utilisés par adminview
    - `views/` : les templates HTML grâce auxquels sont générées les pages renvoyées par adminview
  - [`graphql/`](./src/graphql) : le coeur du backend, c'est là qu'on définit comment répondre à des requêtes GraphQL
    - [`typeDefs/`](./src/graphql/typeDefs) : la définition du schéma GraphQL
    - `connectors/` : (@akka vodol tu peux expliquer ici ?)
    - `resolvers/` : (@akka vodol tu peux expliquer ici ?)
  - [`ldap`](./src/ldap) : gestion des requêtes au LDAP. Fournit une couche d'abstraction permettant aux resolvers GraphQL de ne pas se soucier des spécifités et de la complexité syntaxique des requêtes LDAP


## Base de données

Sigma s'appuie sur deux bases de données (toutes deux hébergées par le BR) :
- Le LDAP frankiz, qui contient identifiants, mots de passe, et toutes les informations sur les utilisateurs et groupes spécifiques X.
  - On y accède par des requêtes LDAP, d'où la nécessité de ldap.js (une dépendance npm faisant office de traducteur javascript/LDAP).
- La BDD propre à Sigma, en [PostgreSQL](https://www.postgresql.fr/), contient les groupes et leurs niveaux de visibilité mais peu d'information sur les utilisateurs (seulement son école).
  - Les requêtes à cette BDD sont gérées par [Knex.js](http://knexjs.org), qui permet de construire facilement des requêtes PostgreSQL.

Cette structure peut sembler lourde et redondante mais s'explique par plusieurs raisons :
- Volonté de pouvoir intégrer d'autres écoles simplement
  - Rajouter leurs groupes sur la BDD sigma
  - Ne pas les obliger à rejoindre notre LDAP avec notre structure
- Garder et continuer à utiliser le LDAP frankiz, car il est utilisé par beaucoup d'autres sites X
- Enjeu de sécurité
  - La BDD de sigma est publique et ne comporte rien de compromettant
  - Le LDAP contient des informations plus sensibles (c'est là que toutes les données sur les utilisateurs sont stockées)

### Utiliser la BDD sigma avec *Knex.js*

cf. [le memo knexjs](./memo\ knexjs.md).

Le knexfile.js et le knex_router.js sont dans `./db`. La localisation des dossiers [seeds](./db/seeds) et [migrations](./db/migrations), est spécifiée dans le knexfile.js, en l'occurrence également dans `./db`.

### Interagir directement avec la BDD sigma en *PostgreSQL*

cf. [le memo postgresql](./memo\ postgresql.md).

Pour accéder à la "vraie" BDD, sur roued (le serveur qui héberge sigma), il faut

- se connecter dessus en `ssh roued`
- faire un `sudo -s postgres` pour se faire passer pour l'utilisateur Unix postgres
- se connecter à la BDD via la commande `psql`
- faire les requêtes en SQL (version PostgreSQL) par l'interface ainsi `psql`

### Fonctions LDAP

On peut facilement explorer le LDAP si on est sur le réseau de l'X, avec [JXplorer](http://jxplorer.org/) en entrant `frankiz` dans nouvelle connexion et en allant dans `net`.

### Ne pas se tromper de LDAP...

Il y a deux LDAP à l'X : le LDAP de la DSI et le LDAP de Frankiz (du BR). 

- Le premier, utilisé pour l'authentification sur les services de l'Ecole (mail polytechnique.edu, le Moodle, Synapses...) ne concerne pas du tout sigma.
- C'est le second qu'on utilise, et "le LDAP" dans le contexte de sigma désignera, sauf indication contraire, le LDAP frankiz.

## Authentification

On utilise [passportjs](https://www.npmjs.com/package/passport) pour l'authentification. La [stratégie "ldapauth"](https://www.npmjs.com/package/passport-ldapauth) de passport permet de s'authentifier contre le LDAP frankiz. 

Lorsqu'une authentification est réussie, on crée une session et on l'associe à un _cookie d'authentification_ qu'on donne à l'utilisateur. Ainsi, les prochaines requêtes venant du même utilisateur seront authentifiées par le cookie, sans besoin de refaire l'authentification contre LDAP. 

La gestion des sessions et de la distribution de cookies est gérée par passport, et est décrite dans `app.ts`.

## Panneau d'administration ("adminview")

D'habitude, toutes les requêtes arrivent en HTTP POST : les requêtes GraphQL, et les requêtes de connexion (à /login).

Lorsque le site reçoit des requêtes HTTP GET, il l'interprète comme étant une requête venant du navigateur d'une personne qui souhaite se connecter directement au serveur backend, donc probablement un admin.

L'application principale du backend fait alors appel à une "sous-application", adminview (ceci est possible grâce à Express et son objet express.router).

## Notes sur javascript

La syntaxe adoptée est JavaScript ES6, un standard moderne (2015) de JavaScript. 

### `import` vs `require`
Le standard moderne permet d'importer des dépendances en utilisant le mot-clé `import`, ce que le serveur Node.js ne comprend pas puisque la version 8 de Node ne comprend que le standard ES5 (2009), qui gère les imports avec `require()`.

### `async`/`await`

## Outils de développement

### ESLint

On utilisera ESLint pour standardiser le code : un ensemble de règles de style pour le code sont appliquées, elles sont configurables dans le fichier `.eslintrc.json`. Par exemple, pour l'instant, la config ESLint impose d'utiliser quatre espaces pour les indentations et d'utiliser des points-virgule en fin de ligne.

Il est préférable de l'installer **globalement** avec `npm install -g eslint`. Pour faire valider les fichiers source par ESLint, utiliser `npm run lint` [^lint-script]. L'option `--fix` permet de corriger les fichiers.

[^lint-script]: qui fait appel au script `eslint src/` défini dans le [`package.json`](./package.json)

Sinon, si vous utilisez Atom ou Visual Studio Code pour éditer votre code, il existe des plugins qui font tourner ESLint en _live_ sur le code et vérifient que tout est en ordre.

Pour mieux comprendre ESLint, se référer à sa [doc](https://eslint.org/docs/user-guide/getting-started).

### nodemon

On utilise [`nodemon`](https://nodemon.io/) pour servir l'application.

> "Just use nodemon instead of node to run your code, and now your process will automatically restart when your code changes"

Dans le script `npm run start` on a mis l'option `--watch dist` pour spécifier qu'il n'est nécessaire de reload que lorsque le dist/ est modifié.

> By default nodemon monitors the current working directory. If you want to take control of that option, use the --watch option to choose specific paths.

Rq: comme indiqué dans le README.md, en mode developpement, il est intéressant de combiner le hot-recompile de `webpack --watch` et le hot-reload de `nodemon` : lancer
```bash
npm run watch
```
dans une console, et
```bash
npm run start
```
dans une autre pour avoir un environnement de développement agréable.

## Contact

Le BR 2016, plus particulièrement Wilson Jallet, Guillaume Wang, Quentin Chevalier et Anatole Romon
Le BR 2017, plus particulièrement Olivér Facklam
